@@ -1,17 +1,11 @@
-from django.utils.encoding import python_2_unicode_compatible
-
-from ..models import models
+from django.contrib.gis.db import models
 
 
-@python_2_unicode_compatible
 class NamedModel(models.Model):
     name = models.CharField(max_length=30)
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
     def __str__(self):
         return self.name
@@ -19,6 +13,10 @@ class NamedModel(models.Model):
 
 class City3D(NamedModel):
     point = models.PointField(dim=3)
+    pointg = models.PointField(dim=3, geography=True)
+
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
 
 
 class Interstate2D(NamedModel):
@@ -28,6 +26,9 @@ class Interstate2D(NamedModel):
 class Interstate3D(NamedModel):
     line = models.LineStringField(dim=3, srid=4269)
 
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
+
 
 class InterstateProj2D(NamedModel):
     line = models.LineStringField(srid=32140)
@@ -36,6 +37,9 @@ class InterstateProj2D(NamedModel):
 class InterstateProj3D(NamedModel):
     line = models.LineStringField(dim=3, srid=32140)
 
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
+
 
 class Polygon2D(NamedModel):
     poly = models.PolygonField(srid=32140)
@@ -44,14 +48,14 @@ class Polygon2D(NamedModel):
 class Polygon3D(NamedModel):
     poly = models.PolygonField(dim=3, srid=32140)
 
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
+
 
 class SimpleModel(models.Model):
 
-    objects = models.GeoManager()
-
     class Meta:
         abstract = True
-        required_db_features = ['gis_enabled']
 
 
 class Point2D(SimpleModel):
@@ -61,6 +65,12 @@ class Point2D(SimpleModel):
 class Point3D(SimpleModel):
     point = models.PointField(dim=3)
 
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
+
 
 class MultiPoint3D(SimpleModel):
     mpoint = models.MultiPointField(dim=3)
+
+    class Meta:
+        required_db_features = {'supports_3d_storage'}
